""" Module for all exceptions
"""


class TechreadException(Exception):
    """Base Exception for all Exceptions raised by
    the Techread functionality

    """

    cli_message_header: str = "Techread Error"
    cli_message_body: str = "An error occurred while processing your request"


class UnauthorizedException(TechreadException):
    """Exception that is raised when
    (i) the response code is 403 - Unauthorized, or
    (ii) the requested action was forbidden by the gateway
    """


class UnknownException(TechreadException):
    """A developer's favorite exception"""


class RequestTooLargeException(TechreadException):
    """Raised RequestTooLargeException the request exceeds the maximal
    request size (at the time of writing 6MB).
    """

    cli_message_header: str = "Request Too Large"
    cli_message_body: str = """The request size exceeds the maximal request size of 10MB.

Please check https://docs.werk24.io/limitations/drawing_file_size.html
for the most up-to-date information on the maximal request size.
"""


class ServerException(TechreadException):
    """Exception that is raised, when the server responded in an unexpected
    way.
    """
    cli_message_header: str = "Server Error"
    cli_message_body: str = """A Server Error occurred while processing your request.

This indicates a problem with the server. The Werk24 service team has been notified and
will investigate the issue. Please try again later. If the problem persists, please
contact us at info@werk24.io.
"""

class InsufficientCreditsException(ServerException):
    """Raised when the user has insufficient credits to perform the requested action

    NOTE: It inherits from ServerException to backwards compatibility.
    """

    cli_message_header: str = "Insufficient Credits"
    cli_message_body: str = """You do not have enough credits to perform the requested action.

Please check your account balance and top up your account if necessary.
"""

class BadRequestException(TechreadException):
    """Raised when the request body cannot be interpreted by the server.
    This indicates that
    (i) someone has fiddled with with request body, or
    (ii) the server API version has been updated and the integration tests
        did not catch the problem.
    If you encounter this exception, please let us know
    """


class ResourceNotFoundException(TechreadException):
    """Raised when we encounter a 404"""


class UnsupportedMediaType(TechreadException):
    """Raised when the uploaded file has a format
    that is not supported by the api
    """

    cli_message_header: str = "Unsupported Media Type"
    cli_message_body: str = """The file format you uploaded is not supported by Werk24.
Please check https://docs.werk24.io/limitations/drawing_file_format.html
for a current list of supported file formats.
"""


class LicenseError(TechreadException):
    cli_message_header: str = "License Error"
    cli_message_body: str = """An error occurred while verifying the license information.

    Please ensure that the license information is in a location where it can be found
    by the client. The client is currently looking for the license information in the
    following locations:

    1. The environment variables W24_AUTH_TOKEN
    2. The file werk24_license.txt in the current directory, and
    3. for backwards compatiability in the file .werk24 in the current directory
    """


class SSLCertificateError(TechreadException):
    """Error raised when the SSL certificate is not valid"""

    cli_message_header: str = "SSL Certificate Error"
    cli_message_body: str = """An error occurred while verifying the SSL certificate.

This typically can have two reasons:

1. Your IT department has activated the Packet Inspection in your firewall.
    This is a common practice in companies to prevent employees from accessing
    certain websites.

2. You might have a virus on your computer that is trying to intercept your
    internet traffic.

In both cases, please contact your IT department to resolve the issue.
Please understand, that this topic is outside of Werk24's influence.
"""



import argparse

from werk24.cli import utils
from werk24.exceptions import BadRequestException, UnauthorizedException
from werk24.models.helpdesk import W24HelpdeskTask


async def main(
    args: argparse.Namespace
) -> None:
    """
    Interact with the Help Desk client through the API.

    Args:
    ----
    args(argparse.Namespace): CLI args generated by
        argparse
    """
    if args.command == "create-helpdesk-task":
        await _handle_create_helpdesk_task(args)


async def _handle_create_helpdesk_task(args: argparse.Namespace) -> None:
    """
    Create a helpdesk task.

    Args:
    ----
    args (argparse.Namespace): Namespace of the args.
    """

    # Create the task
    task = W24HelpdeskTask(
        request_id=args.request_id,
        observed_outcome=args.observed_outcome,
        expected_outcome=args.expected_outcome,
        comment=args.comment,
        position=None,
        importance=args.importance
    )

    # Get the client instance and handle
    # potential errors
    client = utils.make_client()
    async with client as session:
        try:
            task = await session.create_helpdesk_task(task)
        except BadRequestException:
            print("BadRequest. Please check the content of your request.")

        except UnauthorizedException:
            print("Unauthorized. Please check your credentials and the request id")
            return

    # Give the user some feedback
    print("created task with id", task.task_id)



from pydantic import BaseModel


class W24Standard(BaseModel):
    """W24 Standard.

    Attributes:
        blurb (str): Blurb of the Standard. This
            indicates the 'standard' way of writing
            the standard, potentially including the part,
            year and month of the publication. Note that
            this information will only be available if
            it was indicated on the drawing.
    """
    blurb: str



from typing import Literal

from werk24.models.property.base import W24Property
from werk24.models.material import W24Material
from typing import List


class W24PropertyMaterial(W24Property):
    """Material Property of the Material.

    Keep in mind that a part can be manufactured from
    multiple materials (e.g., using 2 different Polymers),
    or specify alternative materials.

    NOTE: future implementations will contain
        more information regarding the shape of
        the input material.

    Attributes:
        materials: Nested List of the materials.
            First layer allows AND-connections,
            the second layer OR-connections.
            This way we are able to encode the
            material specification like these:
            MAT1 and (MAT2 or MAT3) - which occurs
            sometimes for polymer-based part.
            The result would be [[MAT1],[MAT2,MAT3]].
    """
    property_type: Literal["MATERIAL"] = "MATERIAL"
    property_subtype: Literal["MATERIAL"] = "MATERIAL"
    materials: List[List[W24Material]]
